#ifndef BLOBFINGERCOUNT_H
#define BLOBFINGERCOUNT_H

#include <QWidget>
#include <QTouchEvent>
#include <QRadialGradient>
#include <QList>
#include <QTimer>
#include <QImage>
#include <QDialog>

class ComaPoint;

/**
\class BlobFingerCount
\brief The Finger-Count Shortcuts Widget
The widget receives touch events and emits the number of fingers and their position.
@author Gilles Bailly, Alexandre Viard
*/
class BlobFingerCount : public QWidget
{
    Q_OBJECT

public:
    /** Set the delay before the command activation signal*/
    static int delayActivate;

    /**
     * Constructor
     * \param _parent the parent widget
     * \param _right wheter the blob correspond to a right or left hand
     */
    BlobFingerCount(QWidget *_parent = 0, bool _right=true);

    /**
     * Default Destructor
     */
    virtual ~BlobFingerCount();

    /**
    * Called by ComaPoint::timeout()
    * \param comapoint
    */
    virtual void remove(ComaPoint*);


protected:
    /**
     * Reimplement the widget function to display information
     */
    virtual void paintEvent ( QPaintEvent *e);

    /**
    * Reimplement event handling to take QTouchEvent into account
    */
    virtual bool event(QEvent * e);

    /**
    * Initialise variabe according to the QTouchEvent
    */
    virtual bool touchBegin( QTouchEvent * e);

    /**
    * Handle the add and remove of fingers
    */
    virtual bool touchUpdate(QTouchEvent *e);

    /**
    * Reinitialise the value and trigger the activated signal
    */
    virtual bool touchEnd(QTouchEvent *e);
    /**
    * return touch points (id and position) useful for direct mapnipulation
    */
    const QList<QTouchEvent::TouchPoint>& getTouchPoints()const{ return previousPoints; };
    
    /**
    * Calculate the barycenter of the different TouchPoint
    * \param points the position of the different figners
    */
    virtual QPoint barycenter(QList<QTouchEvent::TouchPoint>& points);

signals:
    /**
     * Provide the new numbers of fingers added on the surface
     */
    void nbFingerAdded(int);
    
    /**
    * Provide the new numbers of fingers after removal
    */
    void nbFingerRemoved(int);
    
    /**
    * Send the number of fingers when the activation gesture is detected
    */
    void activated(int);
    
    /**
    * Send the new hand position
    */
    void barycenterMoved(const QPoint&);
    
    /**
    * Send the initial hand position
    */
    void initBarycenter(const QPoint&);
    /**
     * Emitted when pan gesture is detected
     */
    void panStart();
    /**
     * Emitted when pan is stopped
     */
    void panStop();
    

protected:
    /**
     * Reset the ComaPoint
     */
    void resetComaList();
    QList<QTouchEvent::TouchPoint> previousPoints;
    int nbPreviousPoints;
    QList<ComaPoint*> comaList;
    bool m_right;
    QTimer m_panTimer;
    bool m_panMode;
  public slots:
    void panTimeout();
};

//----------------------
/**
\class ComaPoint
\brief Store the number of fingers
the object contain the previous and actual number of fingers detected by the system.
@author Gilles Bailly
*/
class ComaPoint : public QObject
{
    Q_OBJECT
public:
    /**
    * Default Constructor
    */
    ComaPoint(int _nbFingerPrevioous, int _nbFinger, BlobFingerCount* b);
    
    /**
    * \return the current number of fingers
    */
    int nbFinger()const {
        return m_nbFinger;
    }
    
    /**
    * \return the previous number of fingers
    */
    int nbFingerPrevious()const {
        return m_nbFingerPrevious;
    }
    
    /**
    * \return the timer
    */
    
    QTimer& timer() {
        return m_Timer;
    }

public slots:
    void timeout();

protected:
    QTimer m_Timer;
    int m_nbFinger;
    int m_nbFingerPrevious;
    BlobFingerCount* m_BlobFingerCount;
};



#endif // BlobFingerCount_H
