//The Class Definition
#include "qgismtwindow.h"
//Local



//Qt
#include <QDebug>
#include <QMessageBox>
#include <QDialog>
#include <QPoint>
#include <QGraphicsView>

//Qgis
#include <qgisinterface.h>
#include <qgisplugin.h>
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qgsmaplayerregistry.h>
#include <qgscoordinatetransform.h>
#include <qgsmaptopixel.h>
#include <qgsrectangle.h>
#include <qgspoint.h>
#include <qgsmapcanvasitem.h>
#include <qgsmapcanvasmap.h>


/**
\class QGisMTWindow
\brief The QGisMT main windows
The main window of the Multi-Touch version of QGis, feature a fullscreen map display,
gesture oriented action.
@author Alexandre Viard
*/
QGisMTWindow::QGisMTWindow(QgisInterface *iface, QWidget* parent): QMainWindow(parent),mIface(iface)
{
    setWindowTitle("QGis MT Plugins");
    QRect sizer = QApplication::desktop()->screenGeometry();
    showFullScreen();
    setWindowOpacity(1);
    
    // Create the Map Canvas
    my_canvas = new QgsMapCanvas(this, "QGisMTCanvas");
    this->setCentralWidget(my_canvas);
    qDebug() << "CANVAS SIZE" << my_canvas->size();
    
    legendDialog = new QDialog(this);
    my_legend = new QGisMTLegend(my_canvas,mIface,legendDialog,"QGisMTLegend");
    my_legend->setVisible(true);
    legendDialog->setModal(false);
    legendDialog->setVisible(false);
    
    my_canvas->enableAntiAliasing(true);
    my_canvas->map()->enableAntiAliasing(true);;
    my_canvas->setCanvasColor(Qt::darkGray);
    my_canvas->freeze(false);
    my_canvas->setVisible(true);
    my_canvas->refresh();
    
    connect(mIface->mapCanvas(),SIGNAL(renderStarting()),my_canvas,SLOT(refresh()));
    connect(this,SIGNAL(geometryTypeChanged(QGis::GeometryType,QGis::GeometryType)),this,SLOT(updateMenu(QGis::GeometryType,QGis::GeometryType)));
    connect(my_legend,SIGNAL(currentLayerChanged(QString*)),this,SLOT(setActiveLayer(QString*)));
    
    glassLayer = new QGisMTGlassLayer(this);
    glassLayer->setGeometry(sizer);
    glassLayer->show();
    glassLayer->setVisible(true);

    toolVoid = new QGisMTToolVoid(my_canvas);
    toolPan = new QGisMTToolPan(my_canvas);
    toolSelect = new QGisMTToolSelect(my_canvas,glassLayer);
    toolZoom = new QGisMTToolZoom(my_canvas);
    currentTool = toolVoid;
    
    // Initialize Menu
    initMenu();
    setMode(NONE);
    
    // FingerCount init
    fcLeft = new BlobFingerCount(this,false);
    fcLeft->setGeometry(0,22+800,sizer.width()/2,sizer.height());
    fcLeft->setMinimumSize(sizer.width()/2,sizer.height());
    fcLeft->setMaximumSize(sizer.width()/2,sizer.height());
    fcLeft->setWindowOpacity(0.10);
    fcLeft->update();

    fcRight = new BlobFingerCount(this,true);
    fcRight->setGeometry(sizer.width()/2,22+800,sizer.width(),sizer.height());
    fcRight->setMinimumSize(sizer.width()/2,sizer.height());
    fcRight->setMaximumSize(sizer.width()/2,sizer.height());
    fcRight->setWindowOpacity(0.10);
    fcRight->update();

    connect(&m_fingerCount,SIGNAL(continuousCommandSelected(QString)),this, SLOT(updateCommandSelected(QString)));
    connect(&m_fingerCount,SIGNAL(continuousCommandActivated(QString)),this, SLOT(updateCommandActivated(QString)));
    connect(&m_fingerCount,SIGNAL(handMoved(const QLine&, const QLine&)), this,SLOT(commandControl(const QLine&,const QLine&)));
    connect(&m_fingerCount,SIGNAL(handPositionInitialized(const QLine&,const QLine&)),this,SLOT(initHandPosition(const QLine&,const QLine&)));
    //connect(&m_fingerCount,SIGNAL(handMoved(const QLine&, const QLine&)), glassLayer,SLOT(drawBarycenter(const QLine&,const QLine&)));
    
    connect(&m_fingerCount,SIGNAL(panActionStart()),this,SLOT(panToolActionStart()));
    connect(&m_fingerCount,SIGNAL(panActionStop()),this,SLOT(panToolActionStop()));
    
    m_fingerCount.setBlobsFC(fcLeft,fcRight);
    fcRight->show();
    fcLeft->show();

}

QGisMTWindow::~QGisMTWindow()
{
    delete fcLeft;
    delete fcRight;
    delete my_legend;
    delete my_canvas;
    delete glassLayer;
    delete toolPan;
    delete toolZoom;
    delete toolSelect;
    delete toolVoid;
    delete legendDialog;
}


void QGisMTWindow::initMenu()
{

  qDebug()<< __PRETTY_FUNCTION__;
    m_fingerCount.setMenuBar(menuBar());
    //-------------------------------------------------------
    QMenu *geometryMenu = m_fingerCount.addMenu("Geometry",1);


    QAction* zoomAct = m_fingerCount.addAction(geometryMenu, "Zoom",1,false);
    zoomAct->setIcon(mIface->actionZoomToSelected()->icon());
    
    QAction* panAct = mIface->actionPan();
    panAct->setCheckable(false);
    m_fingerCount.addAction(geometryMenu, panAct,2,false);
   // m_fingerCount.addAction(geometryMenu,"Annotate",2,false);

    geometryMenu->addSeparator();

    
    
    QAction* zoomMoreAct = mIface->actionZoomIn();
    zoomMoreAct->setCheckable(false);
    m_fingerCount.addAction(geometryMenu, zoomMoreAct,3,true,UP);
    
    
    QAction * resetGeom = mIface->actionZoomFullExtent();
    resetGeom->setCheckable(false);
    m_fingerCount.addAction(geometryMenu, resetGeom,3);
    
    QAction* zoomLessAct =mIface->actionZoomOut();
    zoomLessAct->setCheckable(false);
    m_fingerCount.addAction(geometryMenu, zoomLessAct,3,true,DOWN);
    
    QAction *layerZoomExtentAct = mIface->actionZoomToLayer();
    layerZoomExtentAct->setCheckable(false);
    m_fingerCount.addAction(geometryMenu, layerZoomExtentAct ,4,true);
    
    
    geometryMenu->addSeparator();
    
    QAction* geomMeasureAreaAction = mIface->actionMeasureArea();
    geomMeasureAreaAction->setCheckable(false);
    m_fingerCount.addAction(geometryMenu, geomMeasureAreaAction,5,false,UP);
    
    
    QAction* geomMeasureAction = mIface->actionMeasure();
    geomMeasureAction->setCheckable(false);
    m_fingerCount.addAction(geometryMenu, geomMeasureAction,5,false);
    
    m_fingerCount.addAction(geometryMenu, "Measure Angle",5,false,DOWN);
    
    

   // connect(panAct,SIGNAL(triggered(bool)), toolPan, SLOT(toolActionEnd()));
   // connect(zoomAct,SIGNAL(triggered(bool)),toolZoom , SLOT(toolActionEnd()));

    connect(zoomMoreAct,SIGNAL(triggered(bool)) , this, SLOT(actionZoomIn()));
    connect(zoomLessAct,SIGNAL(triggered(bool)) , this, SLOT(actionZoomOut()));
    connect(resetGeom,SIGNAL(triggered(bool)), this, SLOT(zoomFullExtent()));


    qDebug()<< "GeometryMenu DONE";
    
    //-------------------------------------------------------
    QMenu *layerMenu = m_fingerCount.addMenu("Layer",2);
    
    
    QAction* clearSelectAct = m_fingerCount.addAction(layerMenu, "Clear Selection", 1,true,UP);
    
    QAction* selectAct = mIface->actionSelect();
    selectAct->setCheckable(false);
    m_fingerCount.addAction(layerMenu, selectAct ,1,false);
    
    // Find DeselectAllIcon
    
    QAction* InvertSelectAct = m_fingerCount.addAction(layerMenu, "Invert Selection", 1,true, DOWN);
    layerMenu->addSeparator();

    QAction *layerPropertyAct = mIface->actionLayerProperties();
    
    
    QAction *layerAct = m_fingerCount.addAction(layerMenu, "Choose layer" ,2);
    QAction *layerAttribAct = mIface->actionOpenTable();
    m_fingerCount.addAction(layerMenu, layerAttribAct ,2,true, DOWN);
    layerMenu->addSeparator();
    
    QAction *layerAddVector = mIface->actionAddOgrLayer();
    QAction *layerAddRaster = mIface->actionAddRasterLayer();
    QAction *layerAddWMS = mIface->actionAddWmsLayer();
    
    m_fingerCount.addAction(layerMenu, layerAddRaster,3,true,UP);
    m_fingerCount.addAction(layerMenu, layerAddVector,3,true,DIRECT);
    m_fingerCount.addAction(layerMenu, layerAddWMS,3,true,DOWN);
    
    QAction *layerNewVector = mIface->actionNewVectorLayer();
    
    m_fingerCount.addAction(layerMenu, layerNewVector,4,true);
    
    
    layerMenu->addSeparator();
    QAction *layerCommitAct = m_fingerCount.addAction(layerMenu,"Commit Change",5,true,UP);
    QAction *layerEditAct = m_fingerCount.addAction(layerMenu, "Allow Edit" ,5,true);
    QAction *layerRollBackAct = m_fingerCount.addAction(layerMenu, "Rollback Change" ,5,true,DOWN);
    layerMenu->addSeparator();
    
    

   // connect(selectAct,SIGNAL(triggered(bool)),toolSelect,SLOT(toolActionEnd()));
    connect(clearSelectAct,SIGNAL(triggered(bool)),this,SLOT(clearSelection()));
    connect(InvertSelectAct,SIGNAL(triggered(bool)),this,SLOT(invertSelection()));
    connect(layerAct,SIGNAL(triggered(bool)),this,SLOT(chooseLayer()));
    connect(layerCommitAct, SIGNAL(triggered(bool)),this, SLOT(toggleLayerEdit()));
    connect(layerEditAct, SIGNAL(triggered(bool)),this, SLOT(toggleLayerEdit()));
    connect(layerRollBackAct, SIGNAL(triggered(bool)),this, SLOT(rollbackEdit()));
    connect(layerAttribAct, SIGNAL(triggered(bool)),this, SLOT(showLayerAttrib()));
    connect(layerPropertyAct, SIGNAL(triggered(bool)),this, SLOT(showLayerProperty()));
    connect(layerZoomExtentAct,  SIGNAL(triggered(bool)),this->mIface, SLOT(zoomToActiveLayer()));

    qDebug()<< "LayerMenu DONE";
    //-------------------------------------------------------
    currVLayerGeomType = QGis::UnknownGeometry;
    actionUnknownGeom = new QAction("Need an editable layer",this);
    actionUnknownGeom->setDisabled(true);
    
    editMenu = m_fingerCount.addMenu("Edit",3);
    
    m_fingerCount.addAction(editMenu,"Move Node",1,false,UP);
    m_fingerCount.addAction(editMenu,actionUnknownGeom,1);
    m_fingerCount.addAction(editMenu,"Delete Node",1,false,DOWN);
    
    editMenu->addSeparator();
 
    m_fingerCount.addAction(editMenu,"Undo",2,true,DIRECT);
    m_fingerCount.addAction(editMenu,"Redo",2,true,DOWN);
    
    editMenu->addSeparator();
    QAction* actionCut = mIface->actionCutFeatures();
    actionCut->setCheckable(false);
    m_fingerCount.addAction(editMenu,actionCut,3,true,UP);
    
    QAction* actionPast = mIface->actionPasteFeatures();
    actionPast->setCheckable(false);
    m_fingerCount.addAction(editMenu,actionPast,3);
    QAction* actionCopy = mIface->actionCopyFeatures();
    actionCopy->setCheckable(false);
    m_fingerCount.addAction(editMenu,actionCopy,3,true,DOWN);
    editMenu->addSeparator();
    
    QAction* actionAddRing = mIface->actionAddRing();
    actionAddRing->setCheckable(false);
    m_fingerCount.addAction(editMenu,actionAddRing,4,true,UP);
    
    QAction* actionAddIsland = mIface->actionAddIsland();
    actionAddIsland->setCheckable(false);
    m_fingerCount.addAction(editMenu,actionAddIsland,4);
    
    editMenu->addSeparator();
    
    QAction* actionDelRing = mIface->actionDeleteRing();
    m_fingerCount.addAction(editMenu,actionDelRing,5,true,UP);
    
    QAction* actionDelPart = mIface->actionDeletePart();
    m_fingerCount.addAction(editMenu,actionDelPart,5);
    
    QAction* actionDelSelec = mIface->actionDeleteSelected();
    m_fingerCount.addAction(editMenu,actionDelSelec,5,true,DOWN);
    
    qDebug()<< "ActionMenu DONE";
    //-------------------------------------------------------
    
    QMenu *analyseMenu = m_fingerCount.addMenu("fTools",4);
    
    m_fingerCount.addAction(analyseMenu,"Distance matrix",1,true,UP);
    m_fingerCount.addAction(analyseMenu,"Sum line lengths",1);
    m_fingerCount.addAction(analyseMenu,"Mean coordinate(s)",1,true,DOWN);
    
    analyseMenu->addSeparator();
    
    m_fingerCount.addAction(analyseMenu,"Points in polygon",2,true,UP);
    m_fingerCount.addAction(analyseMenu,"Line intersections",2);
    m_fingerCount.addAction(analyseMenu,"Nearest neighbour analysis",2,true,DOWN);
    
    analyseMenu->addSeparator();
    
    m_fingerCount.addAction(analyseMenu,"Intersect",3,true,UP);
    m_fingerCount.addAction(analyseMenu,"Union",3);
    m_fingerCount.addAction(analyseMenu,"Convex hull(s)",3,true,DOWN);
    
    analyseMenu->addSeparator();
    
    m_fingerCount.addAction(analyseMenu,"Difference",4,true,UP);
    m_fingerCount.addAction(analyseMenu,"Clip",4);
    m_fingerCount.addAction(analyseMenu,"Dissolve",4,true,DOWN);
    
    analyseMenu->addSeparator();
    
    m_fingerCount.addAction(analyseMenu,"Join attributes",5,true,UP);
    m_fingerCount.addAction(analyseMenu,"Join attributes by location",5);
    m_fingerCount.addAction(analyseMenu,"Split vector layer",5,true,DOWN);  
    
    qDebug()<< "FtoolMenu DONE";
    //-------------------------------------------------------

    QMenu *fileMenu = m_fingerCount.addMenu("File",5);
    
    QAction *fileOpenAct = m_fingerCount.addAction(fileMenu, "Open Project" ,1,true);
    fileMenu->addSeparator();
    
    QAction *fileSaveAsAction = mIface->actionSaveProjectAs();
    m_fingerCount.addAction(fileMenu,fileSaveAsAction,2,true,UP);
    
    QAction *fileSaveAction = mIface->actionSaveProject();
    m_fingerCount.addAction(fileMenu,fileSaveAction,2);
    
    QAction *fileSaveImageAction = mIface->actionSaveMapAsImage();
    m_fingerCount.addAction(fileMenu,fileSaveImageAction,2,true,DOWN);
    
    fileMenu->addSeparator();
    QAction *filePrintAction = mIface->actionPrintComposer();
    m_fingerCount.addAction(fileMenu,filePrintAction,3);
    
    QAction * fileProjectPropertyAction = mIface->actionProjectProperties();
    m_fingerCount.addAction(fileMenu,fileProjectPropertyAction,4);
    
    fileMenu->addSeparator();
    
    QAction *fileExitAct = m_fingerCount.addAction(fileMenu, mIface->actionExit() ,5,true,UP);
    QAction *fileReturnAct = m_fingerCount.addAction(fileMenu, "Return to QGis",5,true);

    connect(fileOpenAct,SIGNAL(triggered(bool)),this,SLOT(openProjectFromQGis()));

    connect(fileExitAct,SIGNAL(triggered(bool)),this,SLOT(actionQuit()));
    connect(fileReturnAct,SIGNAL(triggered(bool)),this,SLOT(actionReturnQGis()));
    qDebug()<< "end initMenu";
    //Demo Mode Large Font
    /*QFont demoFont;
    demoFont.setPointSize(27);
    menuBar()->setFont(demoFont);
    geometryMenu->setFont(demoFont);
    layerMenu->setFont(demoFont);
    editMenu->setFont(demoFont);
    analyseMenu->setFont(demoFont);
    fileMenu->setFont(demoFont);*/
}

QgsMapLayer* QGisMTWindow::getActiveLayer()
{
    return my_canvas->currentLayer();
}


QgsVectorLayer* QGisMTWindow::getActiveVectorLayer()
{
    QgsMapLayer* theMapLayer = my_canvas->currentLayer();

    if (!theMapLayer)
    {
        qDebug()<< "no active layer";
        return NULL;
    }
    QgsVectorLayer* theVectorLayer = dynamic_cast<QgsVectorLayer*>(theMapLayer);
    if (!theVectorLayer)
    {
        qDebug()<< "no active layer";
        return NULL;
    }
    return theVectorLayer;

}

QgsRasterLayer* QGisMTWindow::getActiveRasterLayer()
{
    QgsMapLayer* theMapLayer = my_canvas->currentLayer();

    if (!theMapLayer)
    {
        qDebug()<< "no active layer";
        return NULL;
    }
    QgsRasterLayer* theRasterLayer = dynamic_cast<QgsRasterLayer*>(theMapLayer);
    if (!theRasterLayer)
    {
        qDebug()<< "no active layer";
        return NULL;
    }
    return theRasterLayer;

}


void QGisMTWindow::chooseLayer()
{
    legendDialog->setVisible(!(legendDialog->isVisible()));
}

void QGisMTWindow::setZoomLayerExtent()
{
    QgsMapLayer* layer = getActiveLayer();
    qDebug() << "Zoom Extent " << layer->name() ;
    qDebug() << "Layer Extent" << layer->extent().toString();

    my_canvas->setExtent(layer->extent());
    my_canvas->refresh();
}

void QGisMTWindow::showLayerAttrib()
{
    QgsVectorLayer* vlayer = getActiveVectorLayer();
    if (!(vlayer))
    {
        qDebug() << "no active vector layer";
        return ;
    }
    qDebug() << vlayer->name();
}

void QGisMTWindow::showLayerProperty()
{
    mIface->showLayerProperties(getActiveLayer());
}

void QGisMTWindow::toggleLayerEdit()
{
    QgsVectorLayer* vlayer = getActiveVectorLayer();
    if (vlayer)
    {
        if (vlayer->isEditable()) {
            vlayer->commitChanges();
            my_canvas->freeze(true);
            mIface->actionToggleEditing()->trigger();
            mIface->actionToggleEditing()->trigger();
            my_canvas->freeze(false);
            my_canvas->refresh();
        }
        else {
            mIface->actionToggleEditing()->trigger();
        }
    }

}
void QGisMTWindow::rollbackEdit()
{
    QgsVectorLayer* vlayer = getActiveVectorLayer();
    if (vlayer)
    {
        if (vlayer->isEditable()) {

            vlayer->rollBack();
            my_canvas->freeze(true);
            mIface->actionToggleEditing()->trigger();
            mIface->actionToggleEditing()->trigger();
            my_canvas->freeze(false);
            my_canvas->refresh();
        }
    }
}



void QGisMTWindow::commandControl(const QLine& l1, const QLine& l2)
{
    currentTool->toolAction(l1,l2);
}

void QGisMTWindow::initHandPosition(const QLine& l1, const QLine& l2)
{
    leftPos = l1.p1();
    rightPos = l2.p1();
}

void QGisMTWindow::updateCommandSelected(const QString& name)
{
    qDebug()<<"command selected :"<<name;
    qDebug()<<"previous mode " << mode;
    if (!name.compare("Zoom"))
    {
        if (mode == ZOOM)
            return;
        currentTool->toolActionEnd();
        currentTool = toolZoom;
        setMode(ZOOM);
    }
    else if (!name.compare(mIface->actionPan()->text()) )
    {
        if (mode ==PAN)
            return;
        currentTool->toolActionEnd();
        currentTool = toolPan;
        setMode(PAN);
    }
    else if (!name.compare(mIface->actionSelect()->text()))
    {
        if (mode == SELECT)
            return;
        currentTool->toolActionEnd();
        currentTool = toolSelect;
        setMode(SELECT);
    }
    else
    {
        if (mode == NONE)
            return;
        currentTool->toolActionEnd();
        currentTool = toolVoid;
        setMode(NONE);
    }
}

void QGisMTWindow::updateCommandActivated(const QString& name)
{
  currentTool->toolActionEnd();
  currentTool = toolVoid;
  setMode(NONE);
}

float QGisMTWindow::dist(const QPoint& p1, const QPoint& p2)
{
    float res = ((float)p1.x() - (float)p2.x())*((float)p1.x() - (float)p2.x()) + ((float)p1.y() - (float)p2.y())*((float)p1.y() - (float)p2.y()) ;
    return ::sqrt(res);
}

void QGisMTWindow::actionZoomIn()
{
    qDebug() << "Zoom In";

    my_canvas->zoomWithCenter((rightPos.x()+leftPos.x())/2,(rightPos.y()+leftPos.x())/2,true);
}

void QGisMTWindow::actionZoomOut()
{
    qDebug() << "Zoom Out";

    my_canvas->zoomWithCenter((rightPos.x()+leftPos.x())/2,(rightPos.y()+leftPos.x())/2,false);
}

void QGisMTWindow::openProjectFromQGis()
{
    canvasLayer.clear();
    QMap<QString, QgsMapLayer*> layers = QgsMapLayerRegistry::instance()->mapLayers();

    QMap<QString, QgsMapLayer*>::const_iterator i = layers.constBegin();

    while (i != layers.constEnd()) {
        canvasLayer.append(QgsMapCanvasLayer(i.value(), TRUE));
        qDebug()<< i.key() <<i.value()->name();
        ++i;
    }


    my_canvas->setLayerSet(canvasLayer);
    my_canvas->refresh();
    qDebug() << "CANVAS SIZE" << my_canvas->size();

}
void QGisMTWindow::actionQuit()
{
    mIface->actionExit();
    QCoreApplication::exit();
}

void QGisMTWindow::actionReturnQGis()
{
    this->setVisible(false);
}

void QGisMTWindow::setActiveLayer(QgsMapLayer* layer)
{
    qDebug() << "SetActiveLayer";
    if (!layer)
    {
        qDebug() << "no layer";
        return;
    }

    QList<QgsMapLayer*> layers = my_canvas->layers();

    for (int i = 0; i < layers.size(); ++i) {
        qDebug() << layers.at(i)->name();
        if (!layer->name().compare(layers.at(i)->name()))
        {
            qDebug() << "Layer Selected";
            my_canvas->setCurrentLayer(layers.at(i));
            mIface->setActiveLayer(layers.at(i));
        }
    }
    qDebug() << "Layer Change";
}

void QGisMTWindow::setActiveLayer(QString* layerKey)
{
    qDebug() << __PRETTY_FUNCTION__;
    if (!layerKey)
    {
        qDebug() << "no layer";
        return;
    }

    QList<QgsMapLayer*> layers = my_canvas->layers();

    for (int i = 0; i < layers.size(); ++i) {
        if (!layerKey->compare(layers.at(i)->getLayerID()))
        {
            qDebug() << "Layer Selected";
            qDebug() << layers.at(i)->name();
            my_canvas->setCurrentLayer(layers.at(i));
            mIface->setActiveLayer(layers.at(i));
            updateCurrentLayerType(layers.at(i));;
        }
    }
    qDebug() << "Layer Change";
}

void QGisMTWindow::zoomFullExtent()
{
    my_canvas->zoomToFullExtent();
    my_canvas->refresh();
}

void QGisMTWindow::clearSelection()
{
    QgsVectorLayer* cLyr = getActiveVectorLayer();
    if (cLyr)
        cLyr->removeSelection();

}

void QGisMTWindow::invertSelection()
{
    QgsVectorLayer* cLyr = getActiveVectorLayer();
    if (cLyr)
        cLyr->invertSelection();

}

void QGisMTWindow::updateCurrentLayerType(QgsMapLayer* layer)
{
    if (!(layer))
        return;

    QgsVectorLayer* vlayer = dynamic_cast<QgsVectorLayer*>(layer);
    if (!(vlayer))
        return;

    if (vlayer->geometryType() == currVLayerGeomType)
        return;
    emit geometryTypeChanged(vlayer->geometryType(),currVLayerGeomType);
    currVLayerGeomType = vlayer->geometryType();
}

void QGisMTWindow::updateMenu(QGis::GeometryType current, QGis::GeometryType previous)
{
    qDebug() << __PRETTY_FUNCTION__;
    QList<QAction*> actions = editMenu->actions();
    QAction* addCurrentFeature;
    QAction* addPreviousFeature;
    switch (current) {
    case QGis::Point : {
        addCurrentFeature = mIface->actionCapturePoint();
    }break;

    case QGis::Line : {
        addCurrentFeature = mIface->actionCaptureLine();
    }break;

    case QGis::Polygon : {
        addCurrentFeature = mIface->actionCapturePolygon();
    }break;
    case QGis::UnknownGeometry : {
        addCurrentFeature = actionUnknownGeom;
    }break;
    }

    switch (previous) {
    case QGis::Point : {
        addPreviousFeature = mIface->actionCapturePoint();
    }break;

    case QGis::Line : {
        addPreviousFeature = mIface->actionCaptureLine();
    }break;

    case QGis::Polygon : {
        addPreviousFeature = mIface->actionCapturePolygon();
    }break;
    case QGis::UnknownGeometry : {
        addPreviousFeature = actionUnknownGeom;
    }break;
    }
    qDebug() << "Current" <<addCurrentFeature->text();
    qDebug() << "Previous" <<addPreviousFeature->text();
    m_fingerCount.insertAction(mIface->actionDeleteVertex(),editMenu,addCurrentFeature,1,true,DIRECT);
    editMenu->removeAction(addPreviousFeature);

}

void QGisMTWindow::panToolActionStart()
{
   currentTool = toolPan;
   setMode(PAN);
}
void QGisMTWindow::panToolActionStop()
{
  currentTool->toolActionEnd();
  currentTool=toolVoid;
  setMode(NONE);
}




#ifdef USECMAKE
#include "qgismtwindow.moc"
#endif
